@@ -3,7 +3,7 @@ import logging
 import os
 
 from andes.utils.misc import elapsed
-from andes.io import xlsx, dome, psse   # NOQA
+from andes.io import xlsx, psse   # NOQA
 
 
 logger = logging.getLogger(__name__)
@@ -13,10 +13,9 @@ logger = logging.getLogger(__name__)
 # Typically, column based formats, such as IEEE CDF and PSS/E RAW, are faster to parse
 
 input_formats = {
-    'dome': 'dm',
-    'xlsx': 'xlsx',
-    'matpower': 'm',
-    'psse': ['raw', 'dyr'],
+    'xlsx': ('xlsx',),
+    'matpower': ('m', ),
+    'psse': ('raw', 'dyr'),
 }
 
 # Output formats is a dictionary of supported output formats and their extensions
@@ -27,7 +26,19 @@ output_formats = ['']
 
 def guess(system):
     """
-    input format guess function. First guess by extension, then test by lines
+    Guess the input format based on extension and content.
+
+    Also stores the format name to `system.files.input_format`.
+
+    Parameters
+    ----------
+    system : System
+        System instance with the file name set to `system.files`
+
+    Returns
+    -------
+    str
+        format name
     """
     files = system.files
     maybe = []
@@ -35,52 +46,44 @@ def guess(system):
         maybe.append(files.input_format)
     # first, guess by extension
     for key, val in input_formats.items():
-        if isinstance(val, list):
-            for item in val:
-                if files.ext.strip('.').lower() == item:
-                    maybe.append(key)
-        else:
-            if files.ext.strip('.').lower() == val:
-                maybe.append(key)
+        if files.ext.strip('.').lower() in val:
+            maybe.append(key)
 
     # second, guess by lines
     true_format = ''
     with open(files.case, 'r') as fid:
         for item in maybe:
-            try:
-                parser = importlib.import_module('.' + item, __name__)
-                testlines = getattr(parser, 'testlines')
-                if testlines(fid):
-                    true_format = item
-                    break
-            except ImportError:
-                logger.debug('Parser for {:s} format is not found. '
-                             'Format guess will continue.'.format(item))
+            parser = importlib.import_module('.' + item, __name__)
+            testlines = getattr(parser, 'testlines')
+            if testlines(fid):
+                true_format = item
+                files.input_format = true_format
+                logger.debug(f'Input format guessed as {true_format}.')
+                break
 
-    if true_format:
-        logger.debug('Input format guessed as {:s}.'.format(true_format))
-    else:
+    if not true_format:
         logger.error('Unable to determine case format.')
 
-    files.input_format = true_format
-
     # guess addfile format
     if files.addfile:
         _, add_ext = os.path.splitext(files.addfile)
         for key, val in input_formats.items():
-            if isinstance(val, list):
-                if add_ext[1:] in val:
-                    files.add_format = key
-            else:
-                if add_ext[1:] == val:
-                    files.add_format = key
+            if add_ext[1:] in val:
+                files.add_format = key
+                logger.debug(f'Addfile format guessed as {key}.')
+                break
 
     return true_format
 
 
 def parse(system):
     """
-    Parse input file with the given format in system.files.input_format
+    Parse input file with the given format in `system.files.input_format`.
+
+    Returns
+    -------
+    bool
+        True if successful; False otherwise.
     """
 
     t, _ = elapsed()
@@ -97,7 +100,6 @@ def parse(system):
     # exit if the format parser could not be imported
     try:
         parser = importlib.import_module('.' + input_format, __name__)
-        dmparser = importlib.import_module('.' + 'dome', __name__)
         if add_format:
             addparser = importlib.import_module('.' + add_format, __name__)
     except ImportError:
@@ -123,14 +125,6 @@ def parse(system):
             logger.error('Error parsing addfile {:s} with {:s} parser.'.format(system.files.addfile, input_format))
             return False
 
-    # Try parsing the dynfile with dm parser
-    if system.files.dynfile:
-        logger.info('Parsing input file {:s}.'.format(
-            system.files.dynfile))
-        if not dmparser.read(system, system.files.case):
-            logger.error('Error parsing dynfile {:s} with dm format parser.'.format(system.files.dynfile))
-            return False
-
     _, s = elapsed(t)
     logger.info(f'Input file {system.files.fullname} parsed in {s}.')
 
@@ -138,10 +132,25 @@ def parse(system):
 
 
 def dump(system, output_format):
+    """
+    Dump the System data into the requested output format.
+
+    Parameters
+    ----------
+    system
+        System object
+    output_format : str
+        Output format name. 'xlsx' will be used if is not an instance of `str`.
+
+    Returns
+    -------
+    bool
+        True if successful; False otherwise.
+    """
     if system.files.no_output:
         return
 
-    if output_format is None:
+    if not isinstance(output_format, str):
         output_format = 'xlsx'
 
     outfile = system.files.dump
@@ -152,5 +161,7 @@ def dump(system, output_format):
     _, s = elapsed(t)
     if ret:
         logger.info(f'Converted file {system.files.dump} written in {s}.')
+        return True
     else:
-        logger.error('Format conversion aborted.')
+        logger.error('Format conversion failed.')
+        return False
