@@ -57,7 +57,7 @@ class Node(NodeData, Model):
         NodeData.__init__(self)
         Model.__init__(self, system=system, config=config)
 
-        self.config.add(OrderedDict((('flat_start', 0.0),
+        self.config.add(OrderedDict((('flat_start', 0),
                                      )))
 
         self.group = 'DCTopology'
