@@ -51,6 +51,8 @@ class TDSData(object):
         else:
             raise NotImplementedError(f'Unknown mode {self._mode}.')
 
+        self._latex_warn = True
+
     def _process_names(self):
         self.file_name, _ = os.path.splitext(self.file_name_full)
         self._npy_file = os.path.join(self._path, self.file_name + '.npy')
@@ -236,7 +238,7 @@ class TDSData(object):
     def plot(self, yidx, xidx=(0,), a=None, ycalc=None,
              left=None, right=None, ymin=None, ymax=None, ytimes=None,
              xlabel=None, ylabel=None, legend=True, grid=False,
-             latex=True, dpi=200, savefig=None, show=True, use_bqplot=False, **kwargs):
+             latex=True, dpi=150, savefig=None, show=True, use_bqplot=False, **kwargs):
         """
         Entery function for plot scripting. This function retrieves the x and y values based
         on the `xidx` and `yidx` inputs and then calls `plot_data()` to do the actual plotting.
@@ -417,6 +419,13 @@ class TDSData(object):
 
         using_latex = set_latex(latex)
 
+        if using_latex and self._latex_warn:
+            logger.info('Using LaTeX for rendering.')
+            logger.info('If the rendering takes too long or an error occurs:')
+            logger.info('a) If you are using `andes plot`, disable with optino "-d",')
+            logger.info('b) If you are using `plot()`, set "latex=False".')
+            self._latex_warn = False
+
         # set default x min based on simulation time
         if not left:
             left = xdata[0] - 1e-6
@@ -427,9 +436,11 @@ class TDSData(object):
             line_styles = ['-', '--', '-.', ':']
         line_styles = line_styles * int(len(ydata) / len(line_styles) + 1)
 
+        hold = True
         if not (fig and ax):
             fig = plt.figure(dpi=dpi)
             ax = plt.gca()
+            hold = False
 
         if greyscale:
             plt.gray()
@@ -456,8 +467,12 @@ class TDSData(object):
                 ax.set_ylabel(ylabel)
 
         ax.ticklabel_format(useOffset=False)
-        ax.set_xlim(left=left, right=right)
-        ax.set_ylim(ymin=ymin, ymax=ymax)
+
+        if not hold:
+            ax.set_xlim(left=left, right=right)
+            ax.set_ylim(bottom=ymin, top=ymax)
+        else:
+            ax.autoscale(axis='y')
 
         if grid:
             ax.grid(b=True, linestyle='--')
@@ -605,7 +620,7 @@ def eig_plot(name, args):
         pass
 
 
-def tdsplot(filename, y, x=(0,), tocsv=False, **kwargs):
+def tdsplot(filename, y, x=(0,), tocsv=False, find=None, exclude=None, **kwargs):
     """
     TDS plot main function based on the new TDSData class
 
@@ -629,6 +644,12 @@ def tdsplot(filename, y, x=(0,), tocsv=False, **kwargs):
         if tocsv is True:
             tds_data.export_csv()
             return
+        if find is not None:
+            print(tds_data.find(query=find, exclude=exclude))
+            return
+        if len(y) == 0:
+            logger.error('Must specify Y indices to plot.')
+            return
         y_num = parse_y(y, lower=0, upper=tds_data.nvars)
         tds_data.plot(xidx=x, yidx=y_num, **kwargs)
         return tds_data
