@@ -17,7 +17,7 @@ class PFlow(BaseRoutine):
                                      ('max_iter', 25),
                                      ('report', 1),
                                      )))
-        self.models = system.get_models_with_flag('pflow')
+        self.models = system.find_models('pflow')
 
         self.converged = False
         self.inc = None
@@ -84,7 +84,7 @@ class PFlow(BaseRoutine):
         logger.info('-> Power flow calculation with Newton Raphson method:')
         self._initialize()
         if system.dae.m == 0:
-            logger.error("Loaded case file contains no element.")
+            logger.error("Loaded case contains no power flow element.")
             return False
 
         t0, _ = elapsed()
@@ -98,6 +98,10 @@ class PFlow(BaseRoutine):
                 break
             elif self.niter > self.config.max_iter:
                 break
+            elif np.isnan(mis).any():
+                logger.error(f'NaN found in solution. Convergence not likely')
+                self.niter = self.config.max_iter + 1
+                break
             elif mis > 1e4 * self.mis[0]:
                 logger.error('Mismatch increased too fast. Convergence not likely.')
                 break
