@@ -1,19 +1,21 @@
+import sys
+import os
+import importlib
 from collections import OrderedDict
 
 from andes.routines.base import BaseRoutine
 from andes.utils.misc import elapsed, is_notebook
 from andes.shared import tqdm, np
 from andes.shared import matrix, sparse, spdiag
-from andes.shared import newton_krylov, fsolve, solve_ivp, odeint
-
-from scipy.optimize.nonlin import NoConvergence
 
 import logging
 logger = logging.getLogger(__name__)
 
 
 class TDS(BaseRoutine):
-
+    """
+    Time-domain simulation routine.
+    """
     def __init__(self, system=None, config=None):
         super().__init__(system, config)
         self.config.add(OrderedDict((('tol', 1e-4),
@@ -22,13 +24,14 @@ class TDS(BaseRoutine):
                                      ('fixt', 1),
                                      ('tstep', 1/30),  # recommended step size
                                      ('max_iter', 15),
+                                     ('shrinkt', 1),    # shrink step size if `max_iter` is reached
                                      )))
         # overwrite `tf` from command line
         if system.options.get('tf') is not None:
             self.config.tf = system.options.get('tf')
 
-        self.tds_models = system.get_models_with_flag('tds')
-        self.pflow_tds_models = system.get_models_with_flag(('tds', 'pflow'))
+        self.tds_models = system.find_models('tds')
+        self.pflow_tds_models = system.find_models(('tds', 'pflow'))
 
         # to be computed
         self.deltat = 0
@@ -37,6 +40,7 @@ class TDS(BaseRoutine):
         self.h = 0
         self.next_pc = 0
 
+        # internal status
         self.converged = False
         self.busted = False
         self.niter = 0
@@ -44,8 +48,8 @@ class TDS(BaseRoutine):
         self._last_switch_t = -999  # the last critical time
         self.mis = []
         self.pbar = None
+        self.callpert = None
         self.plotter = None
-
         self.initialized = False
 
     def _initialize(self):
@@ -73,13 +77,16 @@ class TDS(BaseRoutine):
         system.store_switch_times(self.tds_models)
         self.initialized = self.test_initialization()
 
+        self._load_pert()
         _, s1 = elapsed(t0)
+
         if self.initialized is True:
             logger.info(f"Initialization successful in {s1}.")
         else:
             logger.info(f"Initialization error in {s1}.")
+
         if system.dae.n == 0:
-            logger.warning('No dynamic component loaded.')
+            tqdm.write('No dynamic component loaded.')
         return system.dae.xy
 
     def summary(self):
@@ -91,11 +98,13 @@ class TDS(BaseRoutine):
 
         """
         out = list()
-        fixed_or_variable = 'fixed' if self.config.fixt == 1.0 else 'variable'
-        out.append('-> Time Domain Simulation:')
-        out.append(f'Method: {self.config.sparselib}')
-        out.append(f'Simulation time: {self.config.t0}-{self.config.tf}s, '
-                   f'{fixed_or_variable} step h={self.config.tstep}s')
+        out.append('-> Time Domain Simulation Summary:')
+        out.append(f'Sparse Solver: {self.config.sparselib.upper()}')
+        out.append(f'Simulation time: {self.config.t0}-{self.config.tf}s')
+        if self.config.fixt == 1:
+            out.append(f'Fixed step size: h={self.config.tstep:.4g}s')
+        else:
+            out.append(f'Variable step size: h0={self.config.tstep:.4g}s')
 
         out_str = '\n'.join(out)
         logger.info(out_str)
@@ -115,14 +124,18 @@ class TDS(BaseRoutine):
 
         self.summary()
         self._initialize()
-        self.pbar = tqdm(total=100, ncols=70, unit='%')
+        self.pbar = tqdm(total=100, ncols=70, unit='%', file=sys.stdout)
 
         t0, _ = elapsed()
         while (system.dae.t < self.config.tf) and (not self.busted):
             if self.calc_h() == 0:
-                logger.error("Time step calculated to zero. Simulation terminated.")
+                self.pbar.close()
+                logger.error(f"Simulation terminated at t={system.dae.t:.4f}.")
                 break
 
+            if self.callpert is not None:
+                self.callpert(dae.t, system)
+
             if self._implicit_step():
                 # store values
                 dae.ts.store_txyz(dae.t, dae.xy, self.system.get_z(models=self.pflow_tds_models))
@@ -151,14 +164,16 @@ class TDS(BaseRoutine):
             self.load_plotter()
 
     def load_plotter(self):
+        """
+        Manually load a plotter into ``TDS.plotter``.
+        """
         from andes.plot import TDSData  # NOQA
         self.plotter = TDSData(mode='memory', dae=self.system.dae)
 
     def test_initialization(self):
         """
-        Update f and g to see if initialization is successful
+        Update f and g to see if initialization is successful.
         """
-
         system = self.system
         system.e_clear(models=self.pflow_tds_models)
         system.l_update_var(models=self.pflow_tds_models)
@@ -173,10 +188,10 @@ class TDS(BaseRoutine):
             logger.debug('Initialization tests passed.')
             return True
         else:
-            logger.warning('Suspect initialization issue!')
+            logger.error('Suspect initialization issue!')
             fail_idx = np.where(abs(system.dae.fg) >= self.config.tol)
             fail_names = [system.dae.xy_name[int(i)] for i in np.ravel(fail_idx)]
-            logger.warning(f"Check variables {', '.join(fail_names)}")
+            logger.error(f"Check variables {', '.join(fail_names)}")
             return False
 
     def _implicit_step(self):
@@ -205,7 +220,6 @@ class TDS(BaseRoutine):
 
         while True:
             system.e_clear(models=self.pflow_tds_models)
-
             system.l_update_var(models=self.pflow_tds_models)
             system.f_update(models=self.pflow_tds_models)
             system.g_update(models=self.pflow_tds_models)
@@ -235,6 +249,7 @@ class TDS(BaseRoutine):
 
             # check for np.nan first
             if np.isnan(inc).any():
+                self.pbar.close()
                 logger.error(f'NaN found in solution. Convergence not likely')
                 self.niter = self.config.max_iter + 1
                 self.busted = True
@@ -263,6 +278,7 @@ class TDS(BaseRoutine):
                              f'({system.dae.xy_name[np.argmax(inc)]})')
                 break
             if mis > 1000 and (mis > 1e8 * self.mis[0]):
+                self.pbar.close()
                 logger.error(f'Error increased too quickly. Convergence not likely.')
                 self.busted = True
                 break
@@ -317,21 +333,30 @@ class TDS(BaseRoutine):
         if system.dae.t == 0:
             return self._calc_h_first()
 
-        if self.converged:
-            if self.niter >= 15:
-                self.deltat = max(self.deltat * 0.5, self.deltatmin)
-            elif self.niter <= 6:
-                self.deltat = min(self.deltat * 1.1, self.deltatmax)
-            else:
-                self.deltat = max(self.deltat * 0.95, self.deltatmin)
-
-            # adjust fixed time step if niter is high
-            if config.fixt:
-                self.deltat = min(config.tstep, self.deltat)
-        else:
-            self.deltat *= 0.9
-            if self.deltat < self.deltatmin:
+        if config.fixt and not config.shrinkt:
+            if not self.converged:
                 self.deltat = 0
+                self.pbar.close()
+                logger.error(f"Simulation does not converge for the given step size h={self.config.tstep:.4f}.")
+                logger.error(f"Reduce the step size `tstep`, or set `shrinkt = 1` to let it shrink.")
+        else:
+            if self.converged:
+                if self.niter >= 15:
+                    self.deltat = max(self.deltat * 0.5, self.deltatmin)
+                elif self.niter <= 6:
+                    self.deltat = min(self.deltat * 1.1, self.deltatmax)
+                else:
+                    self.deltat = max(self.deltat * 0.95, self.deltatmin)
+
+                # for converged cases, set step size back to the initial `config.tstep`
+                if config.fixt:
+                    self.deltat = min(config.tstep, self.deltat)
+            else:
+                self.deltat *= 0.9
+                if self.deltat < self.deltatmin:
+                    self.deltat = 0
+                    self.pbar.close()
+                    logger.error(f"Time step calculated to zero. Convergence not likely.")
 
         # last step size
         if system.dae.t + self.deltat > config.tf:
@@ -433,148 +458,21 @@ class TDS(BaseRoutine):
 
         self.initialized = False
 
-    # ==================================================
-    # The following code are NOT fully functional !!!
-    # ==================================================
-
-    def _g_wrapper(self, y=None):
+    def _load_pert(self):
         """
-        Wrapper for algebraic equations for general solver
-
-        Parameters
-        ----------
-        y
-
-        Returns
-        -------
-
+        Load perturbation files to ``self.callpert``.
         """
         system = self.system
-        if y is not None:
-            system.dae.y = y
-        system.vars_to_models()
-        system.e_clear(models=self.pflow_tds_models)
-        system.l_update_var(models=self.pflow_tds_models)
-        system.g_update(models=self.pflow_tds_models)
-        return system.dae.g
+        if system.files.pert:
+            if not os.path.isfile(system.files.pert):
+                logger.warning(f'Pert file not found at <{system.files.pert}>.')
+                return False
 
-    def _solve_g(self, verbose):
-        system = self.system
-        dae = system.dae
-        self.converged = False
-        self.niter = 0
-        self.mis = []
+            sys.path.append(system.files.case_path)
+            _, full_name = os.path.split(system.files.pert)
+            name, ext = os.path.splitext(full_name)
 
-        # check if the next step is critical time
-        if self.is_switch_time():
-            self._last_switch_t = system.switch_times[self._switch_idx]
-            system.switch_action(self.pflow_tds_models)
-
-        while True:
-            system.e_clear(models=self.pflow_tds_models)
-            system.l_update_var(models=self.pflow_tds_models)
-            system.g_update(models=self.pflow_tds_models)
-
-            inc = -matrix(system.dae.g)
-            system.j_update(models=self.pflow_tds_models)
-            inc = self.solver.solve(dae.gy, inc)
-            dae.y += np.ravel(np.array(inc))
-            system.vars_to_models()
-
-            mis = np.max(np.abs(inc))
-            self.mis.append(mis)
-            if verbose:
-                print(f't={dae.t:<.4g}, iter={self.niter:<g}, mis={mis:<.4g}')
-            if mis < self.config.tol:
-                self.converged = True
-                break
-            elif self.niter > self.config.max_iter:
-                raise NoConvergence(f'Convergence not reached after {self.config.max_iter} iterations')
-            elif mis >= 1000 and (mis > 1e4 * self.mis[0]):
-                raise NoConvergence('Mismatch increased too fast. Convergence not likely.')
-
-            self.niter += 1
-
-    def _solve_ivp_wrapper(self, t, x, asolver, verbose):
-        system = self.system
-        dae = self.system.dae
-        # store the values from k-1 (the last step)
-        dae.x = x
-        system.vars_to_models()
-        system.dae.store_yt_single()
-        # set new t must come after `store_xyt`
-        dae.t = t
-
-        # solve for algebraic variables
-        if asolver is None:
-            self._solve_g(verbose)
-        elif asolver == 'fsolve':
-            sol, _, ier, mesg = fsolve(self._g_wrapper, dae.y, full_output=True)
-            if ier != 1:
-                raise NoConvergence(f"Cannot solve algebraic equations, error: \n{mesg}")
-            dae.y = sol
-        elif asolver == 'newton_krylov':
-            dae.y = newton_krylov(self._g_wrapper, dae.y, verbose=verbose)
-        else:
-            raise NotImplementedError(f"Unknown algeb_solver {asolver}")
-
-        system.f_update(models=self.pflow_tds_models)
-        system.l_check_eq(models=self.pflow_tds_models)
-        system.l_set_eq(models=self.pflow_tds_models)
-        system.fg_to_dae()
-
-        return dae.f
-
-    def _run_odeint(self, tspan, x0=None, asolver=None, verbose=False, h=0.05, hmax=0, hmin=0):
-        """
-        Run integration with ``scipy.odeint``.
-
-        Warnings
-        --------
-        Function is NOT working. The time-based switching is not handled correctly.
-        """
-        self._initialize()
-        if x0 is None:
-            x0 = self.system.dae.x
-        times = np.arange(tspan[0], tspan[1], h)
-
-        # build critical time list
-        tcrit = np.hstack([np.linspace(i, i+0.5, 100) for i in self.system.switch_times])
-        ret = odeint(self._solve_ivp_wrapper,
-                     x0,
-                     times,
-                     tfirst=True,
-                     args=(asolver, verbose),
-                     full_output=True,
-                     hmax=hmax,
-                     hmin=hmin,
-                     tcrit=tcrit
-                     )
-
-        # store the last step algebraic variables
-        self.system.dae.store_yt_single()
-        self.system.dae.store_xt_array(ret[0], times)
-        return ret
-
-    def _run_solve_ivp(self, tspan, x0=None, asolver=None, method='RK45', verbose=False):
-        """
-        Run integration with ``scipy.solve_ivp``.
-
-        Warnings
-        --------
-        Function not fully implemented. Discontinuities are not properly handled by this wrapper.
-
-        """
-        self._initialize()
-        if x0 is None:
-            x0 = self.system.dae.x
-        ret = solve_ivp(lambda t, x: self._solve_ivp_wrapper(t, x, asolver, verbose=verbose),
-                        tspan,
-                        x0,
-                        method=method)
-
-        # store the last step algebraic variables
-        self.system.dae.store_yt_single()
-
-        self.system.dae.store_xt_array(np.transpose(ret.y), ret.t)
-        return ret
+            module = importlib.import_module(name)
+            self.callpert = getattr(module, 'pert')
+            logger.info(f'Perturbation file <{system.files.pert}> loaded.')
+            return True
