@@ -10,30 +10,29 @@ import inspect
 from collections import OrderedDict
 from typing import List, Dict, Tuple, Union, Optional
 
-from andes.variables.fileman import FileMan
-from andes.variables.dae import DAE
-from andes.routines import all_routines
 from andes.models import non_jit
-from andes.core.param import BaseParam
-from andes.core.model import Model
-from andes.core.var import ExtVar
-from andes.core.discrete import AntiWindupLimiter
-from andes.core.config import Config
+from andes.variables import FileMan, DAE
+from andes.routines import all_routines
+from andes.utils.tab import Tab
 from andes.utils.paths import get_config_path, get_pkl_path
+from andes.core import Config, BaseParam, Model, ExtVar, AntiWindupLimiter
 
-from andes.shared import np, spmatrix
+from andes.shared import np, spmatrix, jac_names
+logger = logging.getLogger(__name__)
 
-IP_ADD = False
 if hasattr(spmatrix, 'ipadd'):
     IP_ADD = True
-
-
-logger = logging.getLogger(__name__)
+else:
+    IP_ADD = False
 
 
 class System(object):
     """
-    New power system class
+    The ANDES power system class.
+
+    This class stores model and routine instances as attributes.
+    Model and routine attribute names are the same as their class names.
+    For example, the ``Bus`` instance of ``system`` is accessible at ``system.Bus``.
     """
     def __init__(self,
                  case: Optional[str] = None,
@@ -43,51 +42,57 @@ class System(object):
                  **kwargs
                  ):
         self.name = name
-        self.options = {} if options is None else options
+        self.options = {}
+        if options is not None:
+            self.options.update(options)
         if kwargs:
             self.options.update(kwargs)
-        self.calls = OrderedDict()
-        self.models = OrderedDict()
-        self.groups = OrderedDict()
-        self.programs = OrderedDict()
-        self.switch_times = np.array([])
+        self.calls = OrderedDict()         # a dictionary with model names (keys) and their ``calls`` instance
+        self.models = OrderedDict()        # model names and instances
+        self.groups = OrderedDict()        # group names and instances
+        self.routines = OrderedDict()      # routine names and instances
+        self.switch_times = np.array([])   # an array of ordered event switching times
 
         # get and load default config file
+        self._config_path = get_config_path()
+        if config_path:
+            self._config_path = config_path
+        self._config_object = self.load_config(self._config_path)
         self.config = Config(self.__class__.__name__)
-        self._config_path = get_config_path() if not config_path else config_path
-        self._config_from_file = self.load_config(self._config_path)
-        self.config.load(self._config_from_file)
+        self.config.load(self._config_object)
 
         # custom configuration for system goes after this line
         self.config.add(OrderedDict((('freq', 60),
                                      ('mva', 100),
                                      ('store_z', 0),
+                                     ('ipadd', 1),
                                      )))
 
-        self.files = FileMan()
-        self.files.set(case=case, **self.options)
-
+        # file path manager
+        self.files = FileMan(case=case, **self.options)
+        # numerical DAE storage
         self.dae = DAE(system=self)
 
-        # dynamic imports: routine import need to query model flags
+        # dynamic imports of groups, models and routines
         self._group_import()
         self._model_import()
-        self._routine_import()
+        self._routine_import()  # routine imports come after models
 
-        self._models_with_flag = {'pflow': self.get_models_with_flag('pflow'),
-                                  'tds': self.get_models_with_flag('tds'),
-                                  'pflow_and_tds': self.get_models_with_flag(('tds', 'pflow')),
-                                  }
+        self._models_flag = {'pflow': self.find_models('pflow'),
+                             'tds': self.find_models('tds'),
+                             'pflow_tds': self.find_models(('tds', 'pflow')),
+                             }
 
-        # ------------------------------
-        # FIXME: reduce clutter with defaultdict `adders` and `setters`, each with `x`, `y`, `f`, and `g`
-        self.f_adders, self.f_setters = list(), list()
-        self.g_adders, self.g_setters = list(), list()
-
-        self.x_adders, self.x_setters = list(), list()
-        self.y_adders, self.y_setters = list(), list()
+        self._adders = dict(f=list(), g=list(), x=list(), y=list())
+        self._setters = dict(f=list(), g=list(), x=list(), y=list())
         self.antiwindups = list()
-        # ------------------------------
+
+    def _clear_adder_setter(self):
+        """
+        Clear adders and setters storage
+        """
+        self._adders = dict(f=list(), g=list(), x=list(), y=list())
+        self._setters = dict(f=list(), g=list(), x=list(), y=list())
 
     def prepare(self, quick=False):
         """
@@ -103,13 +108,13 @@ class System(object):
             self._generate_pretty_print()
         self._check_group_common()
         self._store_calls()
-        self.dill_calls()
+        self.dill()
 
     def setup(self):
         """
-        Set up system for studies
+        Set up system for studies.
 
-        This function is to be called after all data are added.
+        This function is to be called after adding all device data.
         """
         self.set_address()
         self.set_dae_names()
@@ -121,13 +126,15 @@ class System(object):
 
     def reset(self):
         """
-        Reset to the state after reading data and setup (before power flow)
-
-        Returns
-        -------
+        Reset to the state after reading data and setup (before power flow).
 
+        Warnings
+        --------
+        If TDS is initialized, reset will lead to unpredictable state.
         """
-
+        if self.TDS.initialized is True:
+            logger.error('Reset failed because TDS is initialized. \nPlease reload the test case to start over.')
+            return
         self.dae.reset()
         self._call_models_method('a_reset', models=self.models)
         self.e_clear()
@@ -135,6 +142,11 @@ class System(object):
         self.setup()
 
     def add(self, model, param_dict=None, **kwargs):
+        """
+        Add a device instance for an existing model.
+
+        This methods calls the ``add`` method of `model` and registers the device `idx` to group.
+        """
         if model not in self.models:
             logger.warning(f"<{model}> is not an existing model.")
             return
@@ -152,22 +164,26 @@ class System(object):
         group.add(idx=idx, model=self.__dict__[model])
 
     def set_address(self, models=None):
+        """
+        Set addresses for differential and algebraic variables.
+        """
         if models is None:
-            models = self._models_with_flag['pflow']
+            models = self._models_flag['pflow']
 
         # set internal variable addresses
         for mdl in models.values():
             if mdl.flags['address'] is True:
-                logger.debug(f'{mdl.class_name} addresses exist.')
+                logger.debug(f'{mdl.class_name:10s}: addresses exist.')
+                continue
+            if mdl.n == 0:
                 continue
-
-            collate = mdl.flags['collate']
 
             n = mdl.n
             m0 = self.dae.m
             n0 = self.dae.n
             m_end = m0 + len(mdl.algebs) * n
             n_end = n0 + len(mdl.states) * n
+            collate = mdl.flags['collate']
 
             if not collate:
                 for idx, item in enumerate(mdl.algebs.values()):
@@ -182,7 +198,6 @@ class System(object):
 
             self.dae.m = m_end
             self.dae.n = n_end
-
             mdl.flags['address'] = True
 
         # set external variable addresses
@@ -210,10 +225,11 @@ class System(object):
             self.dae.y_tex_name.extend([''] * (self.dae.m - len(self.dae.y_tex_name)))
 
     def set_dae_names(self, models=None):
-        # store variable names
-
+        """
+        Set variable names for differential and algebraic variables, and discrete flags.
+        """
         if models is None:
-            models = self._models_with_flag['pflow']
+            models = self._models_flag['pflow']
 
         for mdl in models.values():
             mdl_name = mdl.class_name
@@ -238,8 +254,13 @@ class System(object):
                             self.dae.o += 1
 
     def initialize(self, models: Optional[Union[str, List, OrderedDict]] = None):
+        """
+        Initialize the variables in the model.
+
+        For each model in sequence, initializes external services and internal variables.
+        """
         if models is None:
-            models = self._models_with_flag['pflow']
+            models = self._models_flag['pflow']
 
         for mdl in models.values():
             # link externals first
@@ -262,37 +283,35 @@ class System(object):
             self.vars_to_models()
 
     def store_adder_setter(self, models=None):
+        """
+        Store the adders and setters for variables and equations.
+        """
         models = self._get_models(models)
-
-        self.f_adders, self.f_setters = list(), list()
-        self.g_adders, self.g_setters = list(), list()
-
-        self.x_adders, self.x_setters = list(), list()
-        self.y_adders, self.y_setters = list(), list()
+        self._clear_adder_setter()
 
         for mdl in models.values():
             if not mdl.n:
                 continue
             for var in mdl.cache.all_vars.values():
                 if var.e_setter is False:
-                    self.__dict__[f'{var.e_code}_adders'].append(var)
+                    self._adders[var.e_code].append(var)
                 else:
-                    self.__dict__[f'{var.e_code}_setters'].append(var)
+                    self._setters[var.e_code].append(var)
 
                 if var.v_setter is False:
-                    self.__dict__[f'{var.v_code}_adders'].append(var)
+                    self._adders[var.v_code].append(var)
                 else:
-                    self.__dict__[f'{var.v_code}_setters'].append(var)
+                    self._setters[var.v_code].append(var)
             for item in mdl.discrete.values():
                 if isinstance(item, AntiWindupLimiter):
                     self.antiwindups.append(item)
 
     def calc_pu_coeff(self):
         """
-        Calculate per unit conversion factor; store input parameters to `vin`, and perform the conversion
-        Returns
-        -------
+        Perform per unit value conversion.
 
+        This function calculates the per unit conversion factors, stores input parameters to `vin`, and perform
+        the conversion.
         """
         Sb = self.config.mva
 
@@ -360,15 +379,37 @@ class System(object):
                     p.set_pu_coeff(coeff)
 
     def l_update_var(self, models: Optional[Union[str, List, OrderedDict]] = None):
+        """
+        Update variabe-based discrete components.
+
+        This function is evaluated before equation updates.
+        """
         self._call_models_method('l_update_var', models)
 
     def l_check_eq(self, models: Optional[Union[str, List, OrderedDict]] = None):
+        """
+        Update equation-dependent discrete components.
+
+        This function is evaluated after equation updates.
+        """
         self._call_models_method('l_check_eq', models)
 
     def l_set_eq(self, models: Optional[Union[str, List, OrderedDict]] = None):
+        """
+        Force set equations after evaluating equations.
+
+        This function is evaluated afte ``l_check_eq``.
+        Currently, it is only used by anti-windup limiters to record changes.
+        """
         self._call_models_method('l_set_eq', models)
 
     def fg_to_dae(self):
+        """
+        Collect equation values into the DAE arrays.
+
+        Additionally, the function resets the differential equations associated with variables pegged by
+        anti-windup limiters.
+        """
         self._e_to_dae('f')
         self._e_to_dae('g')
 
@@ -379,102 +420,118 @@ class System(object):
                     np.put(self.dae.x, key, val)
 
     def f_update(self, models: Optional[Union[str, List, OrderedDict]] = None):
+        """
+        Call the differential equation update method for each model.
+
+        Notes
+        -----
+        Updated equation values are not collected into DAE after this step.
+        """
         self._call_models_method('f_update', models)
 
     def g_update(self, models: Optional[Union[str, List, OrderedDict]] = None):
+        """
+        Call the algebraic equation update method for each model.
+
+        Notes
+        -----
+        Updated equation values are not collected into DAE after this step.
+        """
         self._call_models_method('g_update', models)
 
     def j_update(self, models: Optional[Union[str, List, OrderedDict]] = None):
+        """
+        Call the Jacobian update method for each model.
+
+        Notes
+        -----
+        Updated Jacobians are reflected in the numerical DAE.
+        """
         models = self._get_models(models)
         self._call_models_method('j_update', models)
 
         self.dae.restore_sparse()
         # collect sparse values into sparse structures
-        for j_name in self.dae.jac_name:
+        for j_name in jac_names:
             j_size = self.dae.get_size(j_name)
 
             for mdl in models.values():
-                for row, col, val in mdl.zip_ijv(j_name):
-                    # TODO: use `spmatrix.ipadd` if available
-                    # TODO: fix `ipadd` to get rid of type checking
-                    if isinstance(val, np.float64):
-                        # Workaround for CVXOPT's handling of np.float64
-                        val = float(val)
-                    if isinstance(val, (int, float)) or len(val) > 0:
-                        try:
-                            self.dae.__dict__[j_name] += spmatrix(val, row, col, j_size, 'd')
-                        except TypeError as e:
-                            logger.error(f'{mdl.class_name}: j_name {j_name}, row={row}, col={col}, val={val}, '
-                                         f'j_size={j_size}')
-                            raise e
+                for rows, cols, vals in mdl.triplets.zip_ijv(j_name):
+                    try:
+                        if self.config.ipadd and IP_ADD:
+                            self.dae.__dict__[j_name].ipadd(vals, rows, cols)
+                        else:
+                            self.dae.__dict__[j_name] += spmatrix(vals, rows, cols, j_size, 'd')
+                    except TypeError as e:
+                        logger.error("Error adding Jacobian triplets to existing sparsity pattern.")
+                        logger.error(f'{mdl.class_name}: j_name {j_name}, row={rows}, col={cols}, val={vals}, '
+                                     f'j_size={j_size}')
+                        raise e
 
     def store_sparse_pattern(self, models: Optional[Union[str, List, OrderedDict]] = None):
+        """
+        Collect and store the sparsity pattern of Jacobian matrices.
+
+        This is a runtime function specific to cases.
+
+        Notes
+        -----
+        For `gy` matrix, always make sure the diagonal is reserved.
+        It is a safeguard if the modeling user omitted the diagonal
+        term in the equations.
+        """
         models = self._get_models(models)
         self._call_models_method('store_sparse_pattern', models)
 
         # add variable jacobian values
-        for j_name in self.dae.jac_name:
+        for jname in jac_names:
             ii, jj, vv = list(), list(), list()
 
-            # for `gy` matrix, always make sure the diagonal is reserved
-            # It is a safeguard if the modeling user omitted the diagonal
-            # term in the equations
-            if j_name == 'gy':
+            if jname == 'gy':
                 ii.extend(np.arange(self.dae.m))
                 jj.extend(np.arange(self.dae.m))
                 vv.extend(np.zeros(self.dae.m))
 
-            # logger.debug(f'Jac <{j_name}>, row={ii}')
-
             for mdl in models.values():
-                row_idx = mdl.row_of(f'{j_name}')
-                col_idx = mdl.col_of(f'{j_name}')
-
-                # logger.debug(f'Model <{name}>, row={row_idx}')
-                ii.extend(row_idx)
-                jj.extend(col_idx)
-                vv.extend(np.zeros(len(np.array(row_idx))))
-
-                # add the constant jacobian values
-                for row, col, val in mdl.zip_ijv(f'{j_name}c'):
+                for row, col, val in mdl.triplets.zip_ijv(jname):
                     ii.extend(row)
                     jj.extend(col)
-
-                    if isinstance(val, (float, int)):
-                        vv.extend(val * np.ones(len(row)))
-                    elif isinstance(val, (list, np.ndarray)):
-                        vv.extend(val)
-                    else:
-                        raise TypeError(f'Unknown type {type(val)} in constant jacobian {j_name}')
+                    vv.extend(np.zeros_like(row))
+                for row, col, val in mdl.triplets.zip_ijv(jname + 'c'):
+                    # process constant Jacobians separately
+                    ii.extend(row)
+                    jj.extend(col)
+                    vv.extend(val * np.ones_like(row))
 
             if len(ii) > 0:
-                ii = np.array(ii).astype(int)
-                jj = np.array(jj).astype(int)
-                vv = np.array(vv).astype(float)
+                ii = np.array(ii, dtype=int)
+                jj = np.array(jj, dtype=int)
+                vv = np.array(vv, dtype=float)
 
-            self.dae.store_sparse_ijv(j_name, ii, jj, vv)
-            self.dae.build_pattern(j_name)
+            self.dae.store_sparse_ijv(jname, ii, jj, vv)
+            self.dae.build_pattern(jname)
 
     def vars_to_dae(self):
         """
         From variables to dae variables
 
-        For adders, only those with `v_str` can set the value. ??????
-
-        Returns
-        -------
-
+        Warnings
+        --------
+        For adders, only those with `v_str` can set the value.
         """
         self.dae.clear_xy()
         self._v_to_dae('x')
         self._v_to_dae('y')
 
     def vars_to_models(self):
-        for var in self.y_adders + self.y_setters:
+        """
+        Send DAE variable values to models
+        """
+        for var in self._adders['y'] + self._setters['y']:
             if var.n > 0:
                 var.v[:] = self.dae.y[var.a]
 
-        for var in self.x_adders + self.x_setters:
+        for var in self._adders['x'] + self._setters['x']:
             if var.n > 0:
                 var.v[:] = self.dae.x[var.a]
 
@@ -487,27 +544,25 @@ class System(object):
 
         Parameters
         ----------
-        v_name
-
-        Returns
-        -------
-
+        v_name : 'x' or 'y'
+            Variable type name
         """
         if v_name not in ('x', 'y'):
             raise KeyError(f'{v_name} is not a valid var name')
 
-        for var in self.__dict__[f'{v_name}_adders']:
+        for var in self._adders[v_name]:
             # NOTE:
             # For power flow, they will be initialized to zero.
             # For TDS initialization, they will remain their value.
             if var.n == 0:
                 continue
-            if (var.v_str is None) and isinstance(var, ExtVar):
+            if isinstance(var, ExtVar) and (var.v_str is None):
                 continue
             if var.owner.flags['initialized'] is False:
                 continue
             np.add.at(self.dae.__dict__[v_name], var.a, var.v)
-        for var in self.__dict__[f'{v_name}_setters']:
+
+        for var in self._setters[v_name]:
             if var.owner.flags['initialized'] is False:
                 continue
             if var.n > 0:
@@ -519,19 +574,16 @@ class System(object):
 
         Parameters
         ----------
-        eq_name
-
-        Returns
-        -------
-
+        eq_name : 'x' or 'y'
+            Equation type name
         """
         if eq_name not in ('f', 'g'):
             raise KeyError(f'{eq_name} is not a valid eq name')
 
-        for var in self.__dict__[f'{eq_name}_adders']:
+        for var in self._adders[eq_name]:
             if var.n > 0:
                 np.add.at(self.dae.__dict__[eq_name], var.a, var.e)
-        for var in self.__dict__[f'{eq_name}_setters']:
+        for var in self._setters[eq_name]:
             if var.n > 0:
                 np.put(self.dae.__dict__[eq_name], var.a, var.e)
 
@@ -554,7 +606,21 @@ class System(object):
 
         return np.concatenate(z_dict)
 
-    def get_models_with_flag(self, flag: Optional[Union[str, Tuple]] = None):
+    def find_models(self, flag: Optional[Union[str, Tuple]] = None):
+        """
+        Find models whose ``flag`` field is True.
+
+        Parameters
+        ----------
+        flag : list, str
+            Flags to find
+
+        Returns
+        -------
+        OrderedDict
+            model name : model instance
+
+        """
         if isinstance(flag, str):
             flag = [flag]
 
@@ -566,8 +632,13 @@ class System(object):
                     break
         return out
 
-    def dill_calls(self):
-        logger.debug("Dumping calls to andes.pkl with dill")
+    def dill(self):
+        """
+        Serialize generated functions in ``System.calls`` with dill.
+
+        The serialized file will be stored to ``~/calls.pkl``.
+        """
+        logger.debug("Dumping calls to calls.pkl with dill")
         import dill
         dill.settings['recurse'] = True
 
@@ -575,24 +646,33 @@ class System(object):
         with open(pkl_path, 'wb') as f:
             dill.dump(self.calls, f)
 
-    def undill_calls(self):
+    def undill(self):
+        """
+        Deserialize the function calls from `calls.pkl` with dill.
+        """
         import dill
         dill.settings['recurse'] = True
 
         pkl_path = get_pkl_path()
-
         if not os.path.isfile(pkl_path):
             self.prepare()
 
         with open(pkl_path, 'rb') as f:
             self.calls = dill.load(f)
         logger.debug(f'System undill: loaded <{pkl_path}> file.')
+
         for name, model_call in self.calls.items():
-            self.__dict__[name].calls = model_call
+            if name in self.__dict__:
+                self.__dict__[name].calls = model_call
 
     def _get_models(self, models):
+        """
+        Helper function for sanitizing the ``models`` input.
+
+        The output is an OrderedDict of model names and instances.
+        """
         if models is None:
-            models = self._models_with_flag['pflow']
+            models = self._models_flag['pflow']
         if isinstance(models, str):
             models = {models: self.__dict__[models]}
         elif isinstance(models, Model):
@@ -610,6 +690,9 @@ class System(object):
         return models
 
     def _call_models_method(self, method: str, models: Optional[Union[str, list, Model, OrderedDict]]):
+        """
+        Wrapper to call model methods.
+        """
         if not isinstance(models, OrderedDict):
             models = self._get_models(models)
         for mdl in models.values():
@@ -617,15 +700,11 @@ class System(object):
 
     def _check_group_common(self):
         """
-        Check if all group common variables and parameters are met
+        Check if all group common variables and parameters are met.
 
         Raises
         ------
         KeyError if any parameter or value is not provided
-
-        Returns
-        -------
-        None
         """
         for group in self.groups.values():
             for item in group.common_params:
@@ -642,11 +721,7 @@ class System(object):
 
     def _collect_ref_param(self):
         """
-        Collect indices into `RefParam` for all models
-
-        Returns
-        -------
-
+        Collect indices into `RefParam` for all models.
         """
         # FIXME: too many safe-checking here. Even the model can be non-existent.
 
@@ -676,6 +751,10 @@ class System(object):
                         uid = dest_model.idx2uid(dest_idx)
                         dest_model.ref_params[n].v[uid].append(model_idx)
 
+    def _generate_pycode_file(self):
+        """Generate empty files for storing lambdified Python code (TODO)"""
+        self._call_models_method('generate_pycode_file', self.models)
+
     def _generate_initializers(self):
         # TODO: consider both JIT and non-JIT models
         self._call_models_method('generate_initializers', self.models)
@@ -694,11 +773,7 @@ class System(object):
 
     def _group_import(self):
         """
-        Import groups defined in `devices/group.py`
-
-        Returns
-        -------
-
+        Import groups defined in `devices/group.py`.
         """
         module = importlib.import_module('andes.models.group')
         for m in inspect.getmembers(module, inspect.isclass):
@@ -715,32 +790,31 @@ class System(object):
 
         Models defined in ``jits`` and ``non_jits`` in ``models/__init__.py``
         will be imported and instantiated accordingly.
-
-        Returns
-        -------
-        None
         """
         # non-JIT models
         for file, cls_list in non_jit.items():
             for model_name in cls_list:
                 the_module = importlib.import_module('andes.models.' + file)
                 the_class = getattr(the_module, model_name)
-                self.__dict__[model_name] = the_class(system=self, config=self._config_from_file)
+                self.__dict__[model_name] = the_class(system=self, config=self._config_object)
                 self.models[model_name] = self.__dict__[model_name]
 
                 # link to the group
                 group_name = self.__dict__[model_name].group
                 self.__dict__[group_name].add_model(model_name, self.__dict__[model_name])
 
+        # *** JIT import code ***
         # import JIT models
         # for file, pair in jits.items():
         #     for cls, name in pair.items():
         #         self.__dict__[name] = JIT(self, file, cls, name)
+        # ***********************
 
     def _routine_import(self):
         """
         Dynamically import routines as defined in ``routines/__init__.py``.
 
+        # *** DOCS FROM OLDER VERSION ***
         The command-line argument ``--routine`` is defined in ``__cli__`` in
         each routine file. A routine instance will be stored in the system
         instance with the name being all lower case.
@@ -749,20 +823,25 @@ class System(object):
         ``routines/pflow.py`` where ``__cli__ = 'pflow'``. The class name
         for the routine should be ``Pflow``. The routine instance will be
         saved to ``PowerSystem.pflow``.
-
-        Returns
-        -------
-        None
+        # *******************************
         """
         for file, cls_list in all_routines.items():
             for cls_name in cls_list:
                 file = importlib.import_module('andes.routines.' + file)
                 the_class = getattr(file, cls_name)
                 attr_name = cls_name
-                self.__dict__[attr_name] = the_class(system=self, config=self._config_from_file)
-                self.programs[attr_name] = self.__dict__[attr_name]
+                self.__dict__[attr_name] = the_class(system=self, config=self._config_object)
+                self.routines[attr_name] = self.__dict__[attr_name]
 
     def store_switch_times(self, models=None):
+        """
+        Store event switching time in a sorted Numpy array at ``System.switch_times``.
+
+        Returns
+        -------
+        array-like
+            self.switch_times
+        """
         models = self._get_models(models)
         out = []
         for instance in models.values():
@@ -777,26 +856,37 @@ class System(object):
         return self.switch_times
 
     def switch_action(self, models=None):
+        """
+        Invoke the actions associated with switch times.
+        """
         models = self._get_models(models)
         for instance in models.values():
             instance.switch_action(self.dae.t)
 
     def _p_restore(self):
         """
-        Restore parameters stored in `pin`
-        Returns
-        -------
-
+        Restore parameters stored in `pin`.
         """
         for model in self.models.values():
             for param in model.num_params.values():
                 param.restore()
 
     def e_clear(self, models: Optional[Union[str, List, OrderedDict]] = None):
+        """
+        Clear DAE equation arrays.
+        """
         self.dae.clear_fg()
         self._call_models_method('e_clear', models)
 
+    def remove_pycapsule(self):
+        """Remove PyCapsule objects in solvers"""
+        for r in self.routines.values():
+            r.solver.remove_pycapsule()
+
     def _store_calls(self):
+        """
+        Collect and store model calls into system.
+        """
         logger.debug("Collecting Model.calls into System.")
         for name, mdl in self.models.items():
             self.calls[name] = mdl.calls
@@ -817,7 +907,8 @@ class System(object):
                 logger.debug("Config: set for System")
 
     def get_config(self):
-        """Get config data from models
+        """
+        Get config data from models
 
         Returns
         -------
@@ -827,7 +918,7 @@ class System(object):
         config_dict = configparser.ConfigParser()
         config_dict[self.__class__.__name__] = self.config.as_dict()
 
-        all_with_config = OrderedDict(list(self.programs.items()) +
+        all_with_config = OrderedDict(list(self.routines.items()) +
                                       list(self.models.items()))
 
         for name, instance in all_with_config.items():
@@ -868,10 +959,6 @@ class System(object):
         file_path : str
             path to the configuration file. The user will be prompted if the
             file already exists.
-
-        Returns
-        -------
-        None
         """
         if file_path is None:
             home_dir = os.path.expanduser('~')
@@ -888,3 +975,28 @@ class System(object):
             conf.write(f)
 
         logger.info(f'Config: written to {file_path}')
+
+    def supported_models(self):
+        """
+        Return the support group names and model names in a table.
+
+        Returns
+        -------
+        str
+            A table-formatted string for the groups and models
+        """
+
+        pairs = list()
+        for g in self.groups:
+            models = list()
+            for m in self.groups[g].models:
+                models.append(m)
+            if len(models) > 0:
+                pairs.append((g, ', '.join(models)))
+
+        tab = Tab(title='Supported Groups and Models',
+                  header=['Group', 'Models'],
+                  data=pairs,
+                  )
+
+        return tab.draw()
