@@ -7,7 +7,7 @@ logger = logging.getLogger(__name__)
 class FileMan(object):
     """Define a File Manager class for System"""
 
-    def __init__(self):
+    def __init__(self, case=None, **kwargs):
         """
         Initialize the output file names.
         For inputs, all absolute paths will be respected; and all relative paths are relative to `input_path`.
@@ -42,9 +42,12 @@ class FileMan(object):
         self.lst = None
         self.eig = None
         self.npy = None
+        self.mat = None
         self.prof = None
         self.prof_raw = None
 
+        self.set(case, **kwargs)
+
     def set(self, case=None, **kwargs):
 
         input_format = kwargs.get('input_format')
@@ -92,6 +95,7 @@ class FileMan(object):
             self.lst = None
             self.eig = None
             self.npy = None
+            self.mat = None
             self.prof = None
             self.prof_raw = None
             self.dump = None
@@ -101,6 +105,7 @@ class FileMan(object):
                 output = add_suffix(self.name, 'out')
             prof = add_suffix(self.name, 'prof')
             eig = add_suffix(self.name, 'eig')
+            mat = add_suffix(self.name, 'As')
 
             self.lst = os.path.join(self.output_path, output + '.lst')
             self.npy = os.path.join(self.output_path, output + '.npy')
@@ -108,6 +113,7 @@ class FileMan(object):
 
             self.eig = os.path.join(self.output_path, eig + '.txt')
             self.prof = os.path.join(self.output_path, prof + '.txt')
+            self.mat = os.path.join(self.output_path, mat + '.mat')
             self.prof_raw = os.path.join(self.output_path, prof + '.prof')
             self.dump = os.path.join(self.output_path, dump + '.xlsx')
 
